const messages = {
  ko_KR: {
    after: `The date must be after: '[PARAM]'`,
    afterOrEqual: `The date must be after or equal to: '[PARAM]'`,
    array: `[FIELD] must be an array`,
    before: `The date must be before: '[PARAM]'`,
    beforeOrEqual: `The date must be before or equal to: '[PARAM]'`,
    boolean: `[FIELD] must be true or false`,
    date: `[FIELD] must be a date`,
    different: `[FIELD] must be different to '[PARAM]'`,
    endingWith: `[FIELD] must end with '[PARAM]'`,
    email: `[FIELD] must be a valid email address`,
    falsy: `[FIELD] must be a falsy value (false, 'false', 0 or '0')`,
    in: `[FIELD] must be one of the following options: [PARAM]`,
    integer: `[FIELD] must be an integer`,
    json: `[FIELD] must be a parsable JSON object string`,
    max: `[FIELD] 값은 [PARAM]이하여야합니다.`,
    min: `[FIELD] must be greater than or equal to [PARAM]`,
    maxLength: `[FIELD] 값은 '[PARAM]'자 까지 허용됩니다.`,
    minLength: `[FIELD] must not be less than '[PARAM]' character length`,
    notIn: `[FIELD] must not be one of the following options: [PARAM]`,
    numeric: `[FIELD] must be numeric`,
    optional: `[FIELD] is optional`,
    regexMatch: `[FIELD] must satisify the regular expression: [PARAM]`,
    required: `[FIELD] 값은 필수입력입니다.`,
    same: `[FIELD] must be '[PARAM]'`,
    startingWith: `[FIELD] must start with '[PARAM]'`,
    string: `[FIELD] must be a string`,
    truthy: `[FIELD] must be a truthy value (true, 'true', 1 or '1')`,
    url: `[FIELD] must be a valid url`,
    uuid: `[FIELD] must be a valid UUID`,
    cellphone: `[FIELD] 값이 휴대전화전호 양식에 맞지 않습니다.`
  },
  en_US: {
    after: `The date must be after: '[PARAM]'`,
    afterOrEqual: `The date must be after or equal to: '[PARAM]'`,
    array: `[FIELD] must be an array`,
    before: `The date must be before: '[PARAM]'`,
    beforeOrEqual: `The date must be before or equal to: '[PARAM]'`,
    boolean: `[FIELD] must be true or false`,
    date: `[FIELD] must be a date`,
    different: `[FIELD] must be different to '[PARAM]'`,
    endingWith: `[FIELD] must end with '[PARAM]'`,
    email: `[FIELD] must be a valid email address`,
    falsy: `[FIELD] must be a falsy value (false, 'false', 0 or '0')`,
    in: `[FIELD] must be one of the following options: [PARAM]`,
    integer: `[FIELD] must be an integer`,
    json: `[FIELD] must be a parsable JSON object string`,
    max: `[FIELD] must be less than or equal to [PARAM]`,
    min: `[FIELD] must be greater than or equal to [PARAM]`,
    maxLength: `[FIELD] must not be greater than '[PARAM]' in character length`,
    minLength: `[FIELD] must not be less than '[PARAM]' character length`,
    notIn: `[FIELD] must not be one of the following options: [PARAM]`,
    numeric: `[FIELD] must be numeric`,
    optional: `[FIELD] is optional`,
    regexMatch: `[FIELD] must satisify the regular expression: [PARAM]`,
    required: `[FIELD] must be present`,
    same: `[FIELD] must be '[PARAM]'`,
    startingWith: `[FIELD] must start with '[PARAM]'`,
    string: `[FIELD] must be a string`,
    truthy: `[FIELD] must be a truthy value (true, 'true', 1 or '1')`,
    url: `[FIELD] must be a valid url`,
    uuid: `[FIELD] must be a valid UUID`,
    cellphone: `[FIELD] 값이 휴대전화전호 양식에 맞지 않습니다.`
  }
};

document.addEventListener("alpine:init", () => {
  Iodine.rule("cellphone", (value) => {
    var expr = new RegExp("^01[016789][1-9]{1}[0-9]{2,3}[0-9]{4}$");
    return expr.test(value);
  });

  let i18n = Alpine.reactive({ locale: "ko_KR" });
  Iodine.setErrorMessages(messages[i18n.locale]);

  Alpine.effect(() => {
    Iodine.setErrorMessages(messages[i18n.locale]);
  });

  function checkValidation(elem) {
    const valid = elem._x_valid;
    if (typeof valid !== "object") return;

    const nextVal = elem.value;
    const rules = valid.rules || [];
    const lables = elem.labels;
    if (lables.length > 0) {
      const arr = [];
      for (let i = 0; i < lables.length; i++) {
        let txt = lables[i].textContent;
        if (txt) arr.push(txt);
      }
      const fieldName = arr.join(" ");
      //const fieldName = labelElem.textContent;
      Iodine.setDefaultFieldName(fieldName);
    }

    const error = Iodine.assert(nextVal, rules);
    Iodine.setDefaultFieldName(null);
    const currentErr = valid.error;
    var isValid = error.valid === true;
    if (currentErr !== error) {
      valid.error = error;
    }
    return isValid;
  }

  function createXValidation() {
    return {
      _infoMap: {},
      get(id) {
        return this._infoMap[id];
      },
      set(id, info) {
        this._infoMap[id] = info;
      },
      remove(id) {
        delete this._infoMap[id];
      },
      checkValidation(elem) {
        return checkValidation(elem);
      }
    };
  }

  function getClosestData(el) {
    const arr = Alpine.closestDataStack(el);
    let result = null;
    arr.some((d) => {
      if (d.$el) {
        result = d;
        return true;
      }
      return false;
    });
    return result;
  }

  Alpine.magic("hasValidError", (el) => {
    return (id) => {
      const sId = id + "";
      const rootData = getClosestData(el);
      if (rootData._x_validation) {
        const validObj = rootData._x_validation.get(id);
        return !validObj.isValid;
      }
      return false;
    };
  });

  Alpine.magic("validationMsg", (el) => {
    return (id) => {
      const sId = id + "";
      const rootData = getClosestData(el);
      let msg = "";
      if (rootData._x_validation) {
        const validObj = rootData._x_validation.get(id);
        if (validObj) {
          if (!validObj.isValid) {
            msg = validObj.msg;
          }
        }
      }
      return msg;
    };
  });

  Alpine.directive(
    "validation",
    (el, { expression }, { evaluateLater, effect, cleanup }) => {
      const rootData = getClosestData(el);
      if (rootData._x_validation === undefined) {
        rootData._x_validation = Alpine.reactive(createXValidation());
      }

      const validObj = (el._x_valid = Alpine.reactive({
        id: "",
        error: {},
        rules: [],
        get msg() {
          return this.error.error || "";
        },
        el,
        isReady: false,
        get isValid() {
          return this.error.valid !== false;
        }
      }));

      let getRules = evaluateLater(expression);

      el.addEventListener("input", () => {
        checkValidation(el);
        el.dataset.validError = !validObj.isValid;
      });

      effect(() => {
        if (el._x_bindings) el._x_bindings.value;
        let newId = el.id;
        if (validObj.id !== newId) {
          if (validObj.id) {
            rootData._x_validation.remove(validObj.id);
            el.dataset.validError = validObj.error.valid;
          }
          validObj.id = el.id;
          rootData._x_validation.set(validObj.id, validObj);
        }
        getRules((rules) => {
          validObj.rules = Alpine.reactive(rules);
          if (validObj.isReady) checkValidation(el);
        });
        validObj.isReady = true;
      });

      cleanup(() => {
        if (rootData._x_validation) {
          rootData._x_validation.remove(validObj.id);
        }
        delete validObj;
      });
    }
  );
});

function vaidatableForm() {
  return {
    checkValidation(elem) {
      if (this._x_validation) {
        return this._x_validation.checkValidation(elem);
      }
      return true;
    },
    checkValidationAll() {
      if (!this._x_validation) return true;
      let isValid = true;
      Object.keys(this._x_validation._infoMap).map((id) => {
        let info = this._x_validation.get(id);
        let elemValid = this.checkValidation(info.el);
        isValid = elemValid && isValid;
      });
      return isValid;
    },
    getFirstValidErrorMessage() {
      if (!this._x_validation) return null;
      let msg = null;
      Object.keys(this._x_validation._infoMap).some((id) => {
        let info = this._x_validation.get(id);
        if (!info.isValid) {
          msg = info.msg;
          return true
        }
        return false;
      });
      return msg;
    }
  };
}
